# I2CN_12 I2C to UART bridge
# This is an IC that is I2C centric 2 versions
# A is an 8 pin version
# B is a 14 pin version that also has ADC and DAC
#
# Firmware version 1.6
# Updated  24 Sept. 2018
#
# The defualt address for these devices is 0x48 (36 7 bit)
class I2cn12:
    def __init__(self,address=36,i2c=None,trig=0x1a):
        self.bus = i2c
        self.adr = address    
        self.trig = trig        
        
    # [1]clear UART buffer
    def clr(self):
        self.bus.i2c(self.adr,[self.trig,self.adr,1],0,0) 

    # [5] returns number of bytes in uart buffer
    def bib(self):
        return self.bus.i2c(self.adr,[self.trig,self.adr,5],1,0)
        
    # [6] gets status flag
    def status(self):
        return self.bus.i2c(self.adr,[self.trig,self.adr,6],1,0)

    # [10] sets temporary baud rate
    def baud(self,b):
        bauds=[0,1200,2400,4800,9600,19200,38400,57600,115200] # starts at 1
        try:
            i = bauds.index(b)
            self.bus.i2c(self.adr,[self.trig,self.adr,10,i],0,0)
        except Exception as e:    
            print('Baud rate not in list {}\r\n error{}'.format(bauds[1:],e))
            
        
    # [40] Sets voltage on DAC
    def dac(self,v):
        self.bus.i2c(self.adr,[self.trig,self.adr,40,v],0,0)
        
    # [41] get adc for channel
    def adc(self,chan): # bytes in uart buffer
        vin = self.bus.i2c(self.adr,[self.trig,self.adr,41],2,0)
        rv = 0
        if len(vin) > 1:
            rv = vin[0] * 256
            return rv + vin[1]

    # [42] set Vref for ADC
    def vref(self,v):
        self.bus.i2c(self.adr,[self.trig,self.adr,42,v],0,0)

    # [50] set output for Pn
    def setp(self,p,v):
        self.bus.i2c(self.adr,[self.trig,self.adr,50,p,v],0,0)

    # [51] gets input from Pn
    def getp(self,v):
        return self.bus.i2c(self.adr,[self.trig,self.adr,51,v],1,0)

    # ============ S Y S T E M =================================================
    #---------------------------------------------------------------------------
    # eeprom reset is designed to work regardless of any setttings
    # it does not relay on any values in the eeprom
    # 0x55 is sent instead for the trigger value
    #---------------------------------------------------------------------------
    def eereset(self):
        self.bus.i2c(0,[0x55],0,0)  # special general call

    #---------------------------------------------------------------------------
    # change i2caddress USE 7 BIT
    #---------------------------------------------------------------------------
    def newaddress(self,addr):
        self.bus.i2c(self.adr,[self.trig,self.adr,60,addr*2],0,0) 

    #---------------------------------------------------------------------------
    # read eeprom single byte at address
    #---------------------------------------------------------------------------
    def eeread(self,addr):
        return self.bus.i2c(self.adr,[self.trig,self.adr,61,addr],1,0)[0]

    #---------------------------------------------------------------------------
    # write eeprom single byte at address
    #---------------------------------------------------------------------------
    def eewrite(self,addr,value):
        self.bus.i2c(self.adr,[self.trig,self.adr,62,addr,value],0,0) 

    #---------------------------------------------------------------------------
    # gets device id 2 bytes
    #---------------------------------------------------------------------------
    def deviceid(self):
        l = self.bus.i2c(self.adr,[self.trig,self.adr,63],2,0)
        return (l[0] * 256) + l[1]
        
    #---------------------------------------------------------------------------
    # gets firemware version 3 bytes
    #---------------------------------------------------------------------------
    def firmware(self):
        l = self.bus.i2c(self.adr,[self.trig,self.adr,64],3,0)
        return ('{}.{}.{}'.format(l[0],l[1],l[2]))

    #---------------------------------------------------------------------------
    # reset
    #---------------------------------------------------------------------------
    def reset(self):
        self.bus.i2c(self.adr,[self.trig,self.adr,66],0,0)

    # sends data to UART as list of bytes (numbers) e.g. [44,45,46,10,13] 
    # list of ord
    def send(self,l):
        self.bus.i2c(self.adr,l,0)

    # sends data to UART as string
    def sends(self,s):
        l = []
        for n in s:
            l.append(ord(n))
        self.bus.i2c(self.adr,l,0)

    # gets data from UAR, UART buffer is 80 bytes, retrieving data that does
    # not exist will return 0
    def recv(self):
        n = self.bib()[0]
        if n > 0:
            return self.bus.i2c(self.adr,[],n)
        else:
            return []

    # receive string
    def recvs(self):
        L = self.recv()
        return ''.join(chr(i) for i in L)

    def setbaud(self,rate):
        a3 = (rate >> 24) & 0xff
        a2 = (rate >> 16) & 0xff
        a1 = (rate >> 8) & 0xff
        a0 = rate & 0xff
        self.eewrite(2,a3)
        self.eewrite(3,a2)
        self.eewrite(4,a1)
        self.eewrite(5,a0)
        self.reset() # to take effect - will show an error but its okay
