#!/usr/bin/env python

# I2CN_12 I2C to UART bridge
# Original MicroPython version modified for RPi Python 2/3 - NR 1-12-18
# This is an IC that is I2C centric 2 versions
# A is an 8 pin version
# B is a 14 pin version that also has ADC and DAC
#
# Firmware version 1.6
# Updated  24 Sept. 2018

# Inserted library i2crpi to handle basic i2c functions NA in RPi Python SMBus - NR 1-12-18
# ++++++ one function i2c access +++++++
# Contributed by Raju and updated by Jim
# June 2018 added clock stectch timeout
#   The clock stretch time out is implemented in c as it is only a one off call
#   and besides I don't know how to do it in python. See i2c_set_tout.c for
#   more details.   
#
# July 4 2018
#   Modified to work in any directory
#
# Use:
# import i2cn12rpi 
# dev = i2crpi.IIC(0x24, timeout=4000) # default works with, bus 1 and BCM2835
# dev.i2c(<adr>,[out,out..],return,optional 0) 
#
import io,os.path
import fcntl
import subprocess
from sys import version_info

I2C_SLAVE=0x0703
_ABSPATH__ = os.path.abspath(__file__)
_I2CFPATH__, _I2CFFILE__ = os.path.split(_ABSPATH__)
I2CSTRETCH = 'i2c_set_tout'
I2CSTRETCHFULL = _I2CFPATH__+'/'+I2CSTRETCH

# ==============================================================================
# i2c_base is taken from the broadcom data sheet for BCn
# timeout Number of SCL clock cycles to wait after the rising edge 
#         of SCL before deciding that the slave is not responding.
#         at 100kHZ 2000 = 20mS (1 cycle is 10uS)
# bus is normally 1, if this is changed then the base address needs changing 
class IIC:
   def __init__(self, adr=0x24, i2c_base=0x20804000, timeout=3000, bus=1):
	  # I2CN12 address set on instantiation, trigger byte hardwired  - NR 1-12-18
      self.adr = adr
      self.trig = 0x1a
      self.fr = io.open("/dev/i2c-"+str(bus), "rb", buffering=0)
      self.fw = io.open("/dev/i2c-"+str(bus), "wb", buffering=0)
      if not os.path.isfile(I2CSTRETCHFULL):
          # no compiled file, try to compile
          print("Building ",I2CSTRETCHFULL)
          subprocess.check_output(["gcc", "-o", I2CSTRETCHFULL, I2CSTRETCHFULL+'.c'])
      if os.path.isfile(I2CSTRETCHFULL):   
          rv = subprocess.check_output(['sudo',I2CSTRETCHFULL,str(i2c_base),str(timeout)])
          if type(rv) == bytes: rv = rv.decode('utf-8') # for verison 3
          print(rv)
      else:
         print("cant build "+I2CSTRETCH+" working without")

   def write(self,adr, data):
      fcntl.ioctl(self.fw, I2C_SLAVE, adr)  
      if type(data) is list:
         data = bytes(data)
      self.fw.write(data)

   # return as list of integers
   def read(self,adr, count):
      fcntl.ioctl(self.fr, I2C_SLAVE, adr)
      l = []
      s = self.fr.read(count)
      if len(s) != 0:
         for n in s:
            if version_info[0] < 3:
                l.append(ord(n))
            else:
                l.append(n)          
      return l

   def close(self):
      self.fw.close()
      self.fr.close()
 
   # If general call is set then that address is used for writing instead
   # of the device address, but the device address is used for reading (if any)
   # e.g dev.i2c(35,[1,2],1) # no general call
   #     dev.i2c(35,[1,2,3],1,0) # using general call
   def i2c(self,adr,inputv,outputn,gc=None):
       if isinstance(inputv,list):
           b = bytes(inputv)
           if gc is not None:
               if version_info[0] < 3:
                   self.write(gc,bytearray(inputv))
               else:
                    self.write(gc,b)
           else: 
               if version_info[0] < 3:   
                  self.write(adr,bytearray(inputv))
               else:
                  self.write(adr,b)
           # do output
           if outputn > 0:              
               return self.read(adr,outputn)
       else:
           print('iic input must be a list')       
 
   # i2cn12.py modified to work with i2crpi library - NR 1-12-18   
   # [1]clear UART buffer
   def clr(self):
      self.i2c(self.adr,[self.trig,self.adr,1],0,0) 

   # [5] returns number of bytes in uart buffer
   def bib(self):
      return self.i2c(self.adr,[self.trig,self.adr,5],1,0)
        
   # [6] gets status flag
   def status(self):
      return self.i2c(self.adr,[self.trig,self.adr,6],1,0)

   # [10] sets temporary baud rate
   def baud(self,b):
      bauds=[0,1200,2400,4800,9600,19200,38400,57600,115200] # starts at 1
      try:
         i = bauds.index(b)
         self.i2c(self.adr,[self.trig,self.adr,10,i],0,0)
      except Exception as e:    
         print('Baud rate not in list {}\r\n error{}'.format(bauds[1:],e))

   # [40] Sets voltage on DAC
   def dac(self,v):
      self.i2c(self.adr,[self.trig,self.adr,40,v],0,0)
        
   # [41] get adc for channel
   def adc(self,chan): # bytes in uart buffer
      vin = self.i2c(self.adr,[self.trig,self.adr,41],2,0)
      rv = 0
      if len(vin) > 1:
         rv = vin[0] * 256
         return rv + vin[1]

   # [42] set Vref for ADC
   def vref(self,v):
      self.i2c(self.adr,[self.trig,self.adr,42,v],0,0)

   # [50] set output for Pn
   def setp(self,p,v):
      self.i2c(self.adr,[self.trig,self.adr,50,p,v],0,0)

   # [51] gets input from Pn
   def getp(self,v):
      return self.i2c(self.adr,[self.trig,self.adr,51,v],1,0)

   # ============ S Y S T E M =================================================
   #---------------------------------------------------------------------------
   # eeprom reset is designed to work regardless of any setttings
   # it does not relay on any values in the eeprom
   # 0x55 is sent instead for the trigger value
   #---------------------------------------------------------------------------
   def eereset(self):
      self.i2c(0,[0x55],0,0)  # special general call

   #---------------------------------------------------------------------------
   # change i2caddress USE 7 BIT
   #---------------------------------------------------------------------------
   def newaddress(self,addr):
      self.i2c(self.adr,[self.trig,self.adr,60,addr*2],0,0) 

   #---------------------------------------------------------------------------
   # read eeprom single byte at address
   #---------------------------------------------------------------------------
   def eeread(self,addr):
      return self.i2c(self.adr,[self.trig,self.adr,61,addr],1,0)[0]

   #---------------------------------------------------------------------------
   # write eeprom single byte at address
   #---------------------------------------------------------------------------
   def eewrite(self,addr,value):
      self.i2c(self.adr,[self.trig,self.adr,62,addr,value],0,0) 

   #---------------------------------------------------------------------------
   # gets device id 2 bytes
   #---------------------------------------------------------------------------
   def deviceid(self):
      l = self.i2c(self.adr,[self.trig,self.adr,63],2,0)
      return (l[0] * 256) + l[1]
        
   #---------------------------------------------------------------------------
   # gets firemware version 3 bytes
   #---------------------------------------------------------------------------
   def firmware(self):
      l = self.i2c(self.adr,[self.trig,self.adr,64],3,0)
      return ('{}.{}.{}'.format(l[0],l[1],l[2]))

   #---------------------------------------------------------------------------
   # reset
   #---------------------------------------------------------------------------
   def reset(self):
      self.i2c(self.adr,[self.trig,self.adr,66],0,0)

   # sends data to UART as list of bytes (numbers) e.g. [44,45,46,10,13] 
   # list of ord
   def send(self,l):
      self.i2c(self.adr,l,0)

   # sends data to UART as string
   def sends(self,s):
      l = []
      for n in s:
         l.append(ord(n))
      self.i2c(self.adr,l,0)

   # gets data from UAR, UART buffer is 80 bytes, retrieving data that does
   # not exist will return 0
   def recv(self):
      n = self.bib()[0]
      if n > 0:
         return self.i2c(self.adr,[],n)
      else:
         return []

   # receive string
   def recvs(self):
      L = self.recv()
      return ''.join(chr(i) for i in L)

   def setbaud(self,rate):
      a3 = (rate >> 24) & 0xff
      a2 = (rate >> 16) & 0xff
      a1 = (rate >> 8) & 0xff
      a0 = rate & 0xff
      self.eewrite(2,a3)
      self.eewrite(3,a2)
      self.eewrite(4,a1)
      self.eewrite(5,a0)
      self.reset() # to take effect - will show an error but its okay
